#include <memory.h>
#include <stdio.h>
#include <stdlib.h>

#include "x86_regs.h"
#include "x86_debug.h"
#include "x86_decode.h"
#include "x86_bios.h"

#include "sysenv.h"


#define OFF(addr)	(((addr) >> 0) & 0xffff)
#define SEG(addr)	(((addr) >> 4) & 0xf000)

#define BIOS_ADAPTOR_ROM_BASE	0xc0000

int pci_slot = -1;


int
main (int argc, char **argv)
{
    SysEnv sys;
    int i;

    memset(&sys, 0, sizeof(sys));
    for (i = 1; i < argc; ++i) {
	if (strcmp(argv[i], "-s") == 0) {
	    pci_slot = atoi(argv[++i]);
#ifdef DEBUG
	} else if (strcmp(argv[i], "-d") == 0) {
	    sys.x86.debug |= DEBUG_DECODE_F;
	} else if (strcmp(argv[i], "-D") == 0) {
	    sys.x86.debug |= DEBUG_DISASSEMBLE_F;
	} else if (strcmp(argv[i], "-t") == 0) {
	    sys.x86.debug |= DEBUG_TRACE_F;
	} else if (strcmp(argv[i], "-m") == 0) {
	    sys.x86.debug |= DEBUG_MEM_TRACE_F;
	} else if (strcmp(argv[i], "-i") == 0) {
	    sys.x86.debug |= DEBUG_IO_TRACE_F;
#endif
	}
    }
    sys_init(&sys);

    if (sys_rdw(&sys, 0xc0000) != 0xaa55) {
	printk("ginit: sorry, there seems to be no BIOS at 0xc0000\n");
    }
    x86_bios_init(&sys);
    sys.x86.R_CS = SEG(0xc0003);
    sys.x86.R_IP = OFF(0xc0003);
    sys.x86.R_SS = SEG(sys.mem_size - 1);
    sys.x86.R_SP = OFF(sys.mem_size - 1);
    x86_exec(&sys);

#if 0
    ((char *) sys.mem_base)[0x4000] = 0xcd;	/* INT 10 */
    ((char *) sys.mem_base)[0x4001] = 0x10;
    ((char *) sys.mem_base)[0x4002] = 0xc3;	/* RET */
    sys.x86.R_AH = 0x12;
    sys.x86.R_AL = 0x01;
    sys.x86.R_BL = 0x32;
    sys.x86.R_CS = SEG(0x04000);
    sys.x86.R_IP = OFF(0x04000);
    sys.x86.R_SS = SEG(sys.mem_size - 1);
    sys.x86.R_SP = OFF(sys.mem_size - 1);
    x86_exec(&sys);
#endif

    return 0;
}
