#include <stdio.h>

#include "x86_bios.h"

#ifdef PCI_SUPPORT
# include <linux/bios32.h>
# include <linux/pci.h>
#endif


void (*bios_intr_tab[256])(SysEnv *m, int intno);


static void
undefined_intr (SysEnv *m, int intno)
{
    printk("x86_bios: interrupt %#x is undefined!\n", intno);
}


static void
int10 (SysEnv *m, int intno)
{
    if (m->x86.R_AH == 0x12 && m->x86.R_BL == 0x32) {
	if (m->x86.R_AL == 0) {
	    /* enable CPU accesses to video memory */
	    sys_outb(m, 0x3c2, sys_inb(m, 0x3cc) | 0x02);
	} else if (m->x86.R_AL == 1) {
	    /* disable CPU accesses to video memory */
	    sys_outb(m, 0x3c2, sys_inb(m, 0x3cc) & ~0x02);
	} else {
	    printk("x86_bios.int10: unknown function AH=0x12, BL=0x32, AL=%#02x\n",
		   m->x86.R_AL);
	}
    } else {
	printk("x86_bios.int10: unknown function AH=%#02x, BL=%#02x\n",
	       m->x86.R_AH, m->x86.R_BL);
    }
}


static void
int1a (SysEnv *m, int intno)
{
    switch (m->x86.R_AX) {
#ifdef PCI_SUPPORT
      case 0xb102:
	/* FIND PCI DEVICE */
	m->x86.R_AH =
	  pcibios_find_device (m->x86.R_DX, m->x86.R_CX, m->x86.R_SI,
			       &m->x86.R_BH, &m->x86.R_BL);
	CONDITIONAL_SET_FLAG((m->x86.R_AH != PCIBIOS_SUCCESSFUL), m, F_CF);
#endif /* PCI_SUPPORT */
      default:
	printk("x86_bios.int1a: unknown function AX=%#04x\n", m->x86.R_AX);
    }
}


void
x86_bios_init (SysEnv *m)
{
    int i;

    for (i = 0; i < 256; ++i) {
	((u32*)m->mem_base)[i] = BIOS_SEG << 16;
	bios_intr_tab[i] = undefined_intr;
    }
    bios_intr_tab[0x10] = int10;
    bios_intr_tab[0x1a] = int1a;
}
